@@ -97,6 +97,7 @@
 #define FB620    2		/* CanoScan FB620S */
 #define FS2710   3		/* CanoScan FS2710S */
 #define FB1200   4		/* CanoScan FB1200S */
+#define IX4015   5		/* IX-4015 */
 
 #ifndef MAX
 #define MAX(A,B)	(((A) > (B))? (A) : (B))
@@ -245,9 +246,9 @@ typedef struct CANON_Info
   SANE_Bool can_focus;			/* has got focus control */
   SANE_Bool can_autoexpose;		/* can do autoexposure by hardware */
   SANE_Bool can_calibrate;		/* has got calibration control */
+  SANE_Bool can_diagnose;		/* has diagnostic command */
   SANE_Bool can_eject;			/* can eject medium */
   SANE_Bool can_mirror;			/* can mirror image by hardware */
-  SANE_Bool is_scsi2;			/* is an SCSI-2 device */
   SANE_Bool is_filmscanner;
   SANE_Bool has_fixed_resolutions;	/* only a finite number possible */
 }
