@@ -371,6 +371,7 @@ putnbyte (unsigned char *pnt, unsigned int value, unsigned int nbytes)
 
 /* ==================================================================== */
 /* page codes used by mode_sense and mode_select */
+#define MS_pc_unk     0x2c /* Used by iX500 */
 #define MS_pc_patch   0x2e /* Patch code scanning */
 #define MS_pc_counter 0x2f /* Page number and counter reset */
 #define MS_pc_autocolor 0x32 /* Automatic color detection */
@@ -537,6 +538,16 @@ putnbyte (unsigned char *pnt, unsigned int value, unsigned int nbytes)
 #define SD_gdi_string                   "GET DEVICE ID   "
 #define SD_gdi_len                      16
 
+#define SD_preread_string               "SET PRE READMODE"
+#define SD_preread_stringlen            16
+#define SD_preread_len                  32
+#define set_SD_preread_xres(in, b)      putnbyte(in + 0x10, b, 2)
+#define set_SD_preread_yres(in, b)      putnbyte(in + 0x12, b, 2)
+#define set_SD_preread_paper_width(sb, val)   putnbyte(sb + 0x14, val, 4)
+#define set_SD_preread_paper_length(sb, val)  putnbyte(sb + 0x18, val, 4)
+#define set_SD_preread_composition(sb, val)   putnbyte(sb + 0x1c, val, 1)
+#define set_SD_preread_escan(sb, val)   putnbyte(sb + 0x1d, val, 1)
+
 /* ==================================================================== */
 /* SET_WINDOW */
 #define SET_WINDOW_code         0x24
@@ -560,6 +571,8 @@ putnbyte (unsigned char *pnt, unsigned int value, unsigned int nbytes)
 #define set_R_datatype_code(sb, val) sb[0x02] = val
 #define R_datatype_imagedata		0x00
 #define R_datatype_pixelsize		0x80
+#define R_datatype_papersize		0x81
+#define R_datatype_effective_id         0x82
 #define set_R_window_id(sb, val)       sb[0x05] = val
 #define set_R_xfer_length(sb, val)     putnbyte(sb + 0x06, val, 3)
 
@@ -572,6 +585,10 @@ putnbyte (unsigned char *pnt, unsigned int value, unsigned int nbytes)
 #define get_PSIZE_req_driv_lut(in)     getbitfield(in + 0x10, 1, 6)
 #define get_PSIZE_req_driv_valid(in)   getbitfield(in + 0x10, 1, 0)
 
+#define R_PAPER_len                0x08
+#define get_PAPER_job_sep(in)          getnbyte(in + 0x02, 1)
+#define get_PAPER_paper_w(in)          getnbyte(in + 0x03, 1)
+
 /* ==================================================================== */
 /* SEND */
 #define SEND_code               0x2a
@@ -582,8 +599,8 @@ putnbyte (unsigned char *pnt, unsigned int value, unsigned int nbytes)
 #define S_datatype_halftone_mask        0x02
 #define S_datatype_gamma_function       0x03*/
 #define S_datatype_lut_data             0x83
-/*#define S_datatype_jpg_q_table          0x88*/
-#define S_datatype_endorser_data       0x90
+#define S_datatype_jpg_q_table          0x88
+#define S_datatype_endorser_data        0x90
 /*#define S_EX_datatype_lut		0x01
 #define S_EX_datatype_shading_data	0xa0
 #define S_user_reg_gamma		0xc0
@@ -608,6 +625,13 @@ putnbyte (unsigned char *pnt, unsigned int value, unsigned int nbytes)
 #define S_lut_data_min_len          256
 #define S_lut_data_max_len          1024
 
+/*q-table*/
+#define S_q_table_header_len   0x0a
+#define S_q_table_y_len        0x40
+#define set_S_q_table_y_len(sb, val)    putnbyte(sb + 4, val, 2)
+#define S_q_table_uv_len       0x40
+#define set_S_q_table_uv_len(sb, val)   putnbyte(sb + 6, val, 2)
+
 /*endorser*/
 #define S_e_data_min_len	18 /*minimum 18 bytes no string bytes*/
 #define S_e_data_max_len	98 /*maximum 18 bytes plus 80 string bytes*/
