@@ -2,7 +2,7 @@
 
    Copyright (C) 2007-2009 Nicolas Martin, <nicols-guest at alioth dot debian dot org>
    Copyright (C) 2008 Dennis Lou, dlou 99 at yahoo dot com
-   Copyright (C) 2011-2012 Rolf Bensch <rolf at bensch hyphen online dot de>
+   Copyright (C) 2011-2013 Rolf Bensch <rolf at bensch hyphen online dot de>
 
    This file is part of the SANE package.
 
@@ -76,14 +76,22 @@
 #define MF4360_PID 0x26ec
 #define D480_PID   0x26ed
 #define MF4320_PID 0x26ee
+#define D420_PID   0x26ef
 #define MF3200_PID 0x2684
 #define MF6500_PID 0x2686
 #define MF4410_PID 0x2737
+#define MF3010_PID 0x2759
+#define MF4770_PID 0x2774
 /* the following are all untested */
 #define MF5630_PID 0x264e
 #define MF5650_PID 0x264f
 #define MF8100_PID 0x2659
+#define MF5880_PID 0x26f9
+#define MF6680_PID 0x26fa
 #define MF8030_PID 0x2707
+#define MF4550_PID 0x2736
+#define MF4570_PID 0x275a
+#define IR1133_PID 0x2742
 
 
 enum iclass_state_t
@@ -119,6 +127,8 @@ typedef struct iclass_t
   unsigned buf_len, blk_len;
 
   unsigned last_block;
+
+  uint8_t generation;           /* New multifunctionals are (generation == 2) */
 } iclass_t;
 
 
@@ -142,7 +152,7 @@ static int
 has_paper (pixma_t * s)
 {
   iclass_t *mf = (iclass_t *) s->subdriver;
-  return (mf->current_status[1] == 0);
+  return ((mf->current_status[1] & 0x0f) == 0);         /* allow 0x10 as ADF paper OK */
 }
 
 static int
@@ -185,10 +195,11 @@ activate (pixma_t * s, uint8_t x)
     case MF4600_PID:
     case MF6500_PID:
     case D480_PID:
+    case D420_PID:
     case MF4360_PID:
+    case MF4100_PID:
       return iclass_exec (s, &mf->cb, 1);
       break;
-    case MF4100_PID:
     default:
       return pixma_exec (s, &mf->cb);
     }
@@ -215,10 +226,11 @@ select_source (pixma_t * s)
     case MF4600_PID:
     case MF6500_PID:
     case D480_PID:
+    case D420_PID:
     case MF4360_PID:
+    case MF4100_PID:
       return iclass_exec (s, &mf->cb, 0);
       break;
-    case MF4100_PID:
     default:
       return pixma_exec (s, &mf->cb);
     }
@@ -248,10 +260,11 @@ send_scan_param (pixma_t * s)
     case MF4600_PID:
     case MF6500_PID:
     case D480_PID:
+    case D420_PID:
     case MF4360_PID:
+    case MF4100_PID:
       return iclass_exec (s, &mf->cb, 0);
       break;
-    case MF4100_PID:
     default:
       return pixma_exec (s, &mf->cb);
     }
@@ -267,10 +280,16 @@ request_image_block (pixma_t * s, unsigned flag, uint8_t * info,
   const int hlen = 2 + 6;
 
   memset (mf->cb.buf, 0, 11);
-  pixma_set_be16 ((s->cfg->pid == MF4410_PID ? cmd_read_image2 : cmd_read_image), mf->cb.buf);
+  pixma_set_be16 (((s->cfg->pid == MF3010_PID ||
+                    s->cfg->pid == MF4410_PID ||
+                    s->cfg->pid == MF4770_PID ||
+                    s->cfg->pid == MF4550_PID) ? cmd_read_image2 : cmd_read_image), mf->cb.buf);
   mf->cb.buf[8] = flag;
   mf->cb.buf[10] = 0x06;
-  expected_len = (s->cfg->pid == MF4410_PID ||
+  expected_len = (s->cfg->pid == MF3010_PID ||
+                  s->cfg->pid == MF4410_PID ||
+                  s->cfg->pid == MF4770_PID ||
+                  s->cfg->pid == MF4550_PID ||
                   s->cfg->pid == MF4600_PID ||
                   s->cfg->pid == MF6500_PID ||
                   s->cfg->pid == MF8030_PID) ? 512 : hlen;
@@ -281,7 +300,10 @@ request_image_block (pixma_t * s, unsigned flag, uint8_t * info,
       *size = pixma_get_be16 (mf->cb.buf + 6);    /* 16bit size */
       error = 0;
 
-      if (s->cfg->pid == MF4410_PID ||
+      if (s->cfg->pid == MF3010_PID ||
+          s->cfg->pid == MF4410_PID ||
+          s->cfg->pid == MF4770_PID ||
+          s->cfg->pid == MF4550_PID ||
           s->cfg->pid == MF4600_PID ||
           s->cfg->pid == MF6500_PID ||
           s->cfg->pid == MF8030_PID)
@@ -304,7 +326,10 @@ read_image_block (pixma_t * s, uint8_t * data, unsigned size)
   int error;
   unsigned maxchunksize, chunksize, count = 0;
   
-  maxchunksize = MAX_CHUNK_SIZE * ((s->cfg->pid == MF4410_PID ||
+  maxchunksize = MAX_CHUNK_SIZE * ((s->cfg->pid == MF3010_PID ||
+                                    s->cfg->pid == MF4410_PID ||
+                                    s->cfg->pid == MF4770_PID ||
+                                    s->cfg->pid == MF4550_PID ||
                                     s->cfg->pid == MF4600_PID ||
                                     s->cfg->pid == MF6500_PID ||
                                     s->cfg->pid == MF8030_PID) ? 4 : 1);
@@ -341,10 +366,11 @@ read_error_info (pixma_t * s, void *buf, unsigned size)
     case MF4600_PID:
     case MF6500_PID:
     case D480_PID:
+    case D420_PID:
     case MF4360_PID:
+    case MF4100_PID:
       error = iclass_exec (s, &mf->cb, 0);
       break;
-    case MF4100_PID:
     default:
       error = pixma_exec (s, &mf->cb);
     }
@@ -388,16 +414,21 @@ static int
 step1 (pixma_t * s)
 {
   int error;
+  iclass_t *mf = (iclass_t *) s->subdriver;
 
   error = query_status (s);
   if (error < 0)
     return error;
   if (s->param->source == PIXMA_SOURCE_ADF && !has_paper (s))
     return PIXMA_ENO_PAPER;
-  if (error >= 0)
-    error = activate (s, 0);
-  if (error >= 0)
-    error = activate (s, 4);
+  /* activate only seen for generation 1 scanners */
+  if (mf->generation == 1)
+    {
+      if (error >= 0)
+        error = activate (s, 0);
+      if (error >= 0)
+        error = activate (s, 4);
+    }
   return error;
 }
 
@@ -448,6 +479,10 @@ iclass_open (pixma_t * s)
   mf->cb.cmd_header_len = 10;
   mf->cb.cmd_len_field_ofs = 7;
 
+  /* set generation = 2 for new multifunctionals */
+  mf->generation = (s->cfg->pid >= MF8030_PID) ? 2 : 1;
+  PDBG (pixma_dbg (3, "*iclass_open***** This is a generation %d scanner.  *****\n", mf->generation));
+
   PDBG (pixma_dbg (3, "Trying to clear the interrupt buffer...\n"));
   if (handle_interrupt (s, 200) == 0)
     {
@@ -475,6 +510,13 @@ iclass_check_param (pixma_t * s, pixma_scan_param_t * sp)
 
   sp->depth = 8;
   sp->line_size = ALIGN_SUP (sp->w, 32) * sp->channels;
+
+  /* Some exceptions here for particular devices */
+  /* Those devices can scan up to Legal 14" with ADF, but A4 11.7" in flatbed */
+  if (sp->source == PIXMA_SOURCE_FLATBED
+      && ( s->cfg->pid == MF4770_PID ))
+    sp->h = MIN (sp->h, 877 * sp->xdpi / 75);
+
   return 0;
 }
 
@@ -512,12 +554,20 @@ iclass_scan (pixma_t * s)
   mf->blk_len = 0;
 
   error = step1 (s);
-  if (error >= 0)
-    error = start_session (s);
-  if (error >= 0)
-    mf->state = state_scanning;
-  if (error >= 0)
-    error = select_source (s);
+  if (error >= 0 && (s->param->adf_pageid == 0 || mf->generation == 1))
+    { /* single sheet or first sheet from ADF */
+      PDBG (pixma_dbg (3, "*iclass_scan***** start scanning *****\n"));
+      error = start_session (s);
+      if (error >= 0)
+        mf->state = state_scanning;
+      if (error >= 0)
+        error = select_source (s);
+    }
+  else if (error >= 0)
+    { /* next sheet from ADF */
+      PDBG (pixma_dbg (3, "*iclass_scan***** scan next sheet from ADF  *****\n"));
+      mf->state = state_scanning;
+    }
   if (error >= 0)
     error = send_scan_param (s);
   if (error >= 0)
@@ -601,7 +651,10 @@ iclass_fill_buffer (pixma_t * s, pixma_imagebuf_t * ib)
       if (n != 0)
         {
           if (s->param->channels != 1 &&
-	          s->cfg->pid != MF4410_PID &&
+                  s->cfg->pid != MF3010_PID &&
+                  s->cfg->pid != MF4410_PID &&
+                  s->cfg->pid != MF4770_PID &&
+	          s->cfg->pid != MF4550_PID &&
 	          s->cfg->pid != MF4600_PID &&
 	          s->cfg->pid != MF6500_PID &&
 	          s->cfg->pid != MF8030_PID)
@@ -648,12 +701,22 @@ iclass_finish_scan (pixma_t * s)
     case state_finished:
       query_status (s);
       query_status (s);
-      activate (s, 0);
-      query_status (s);
-      if (mf->last_block == 0x28 || (s->cfg->pid==MF4410_PID && mf->last_block==0x38))
+      if (mf->generation == 1)
+        { /* activate only seen for generation 1 scanners */
+          activate (s, 0);
+          query_status (s);
+        }
+      /* 0x38 = last block and ADF empty
+       * 0x28 = last block and Paper in ADF */
+      if (mf->last_block==0x38                                  /* ADF empty */
+          || (mf->generation == 1 && mf->last_block == 0x28))   /* generation 1 scanner or Paper in ADF */
 	{
+          PDBG (pixma_dbg (3, "*iclass_finish_scan***** abort session  *****\n"));
 	  abort_session (s);
 	}
+      else
+        PDBG (pixma_dbg (3, "*iclass_finish_scan***** wait for next page from ADF  *****\n"));
+
       mf->state = state_idle;
       /* fall through */
     case state_idle:
@@ -703,7 +766,8 @@ static const pixma_scan_ops_t pixma_iclass_ops = {
             1,                        /* iface */		\
             &pixma_iclass_ops,        /* ops */		\
             dpi, dpi,                 /* xdpi, ydpi */	\
-            0, 0,                     /* ext_min_dpi & ext_max_dpi not used in this subdriver */ \
+            0, 0,                     /* adftpu_min_dpi & adftpu_max_dpi not used in this subdriver */ \
+            0, 0,                     /* tpuir_min_dpi & tpuir_max_dpi not used in this subdriver */   \
             w, h,                     /* width, height */	\
             PIXMA_CAP_GRAY|PIXMA_CAP_EVENTS|cap             \
 }
@@ -711,17 +775,25 @@ const pixma_config_t pixma_iclass_devices[] = {
   DEV ("Canon imageCLASS MF4270", "MF4270", MF4200_PID, 600, 640, 877, PIXMA_CAP_ADF),
   DEV ("Canon imageCLASS MF4150", "MF4100", MF4100_PID, 600, 640, 877, PIXMA_CAP_ADF),
   DEV ("Canon imageCLASS MF4690", "MF4690", MF4600_PID, 600, 640, 877, PIXMA_CAP_ADF),
+  DEV ("Canon imageCLASS D420", "D420", D420_PID, 600, 640, 877, PIXMA_CAP_ADFDUP),
   DEV ("Canon imageCLASS D480", "D480", D480_PID, 600, 640, 877, PIXMA_CAP_ADFDUP),
   DEV ("Canon imageCLASS MF4360", "MF4360", MF4360_PID, 600, 640, 877, PIXMA_CAP_ADFDUP),
   DEV ("Canon imageCLASS MF4320", "MF4320", MF4320_PID, 600, 640, 877, PIXMA_CAP_ADF),
   DEV ("Canon imageCLASS MF4010", "MF4010", MF4010_PID, 600, 640, 877, 0),
   DEV ("Canon imageCLASS MF3240", "MF3240", MF3200_PID, 600, 640, 877, 0),
   DEV ("Canon imageClass MF6500", "MF6500", MF6500_PID, 600, 640, 877, PIXMA_CAP_ADF),
-  DEV ("Canon imageCLASS MF4410", "MF4410", MF4410_PID, 600, 640, 877, 0),
+  DEV ("Canon imageCLASS MF4410", "MF4410", MF4410_PID, 600, 640, 877, PIXMA_CAP_ADF),
+  DEV ("Canon i-SENSYS MF4550d", "MF4550", MF4550_PID, 600, 640, 877, PIXMA_CAP_ADF),
+  DEV ("Canon i-SENSYS MF3010", "MF3010", MF3010_PID, 600, 640, 877, 0),
+  DEV ("Canon imageCLASS MF4770n", "MF4770", MF4770_PID, 600, 640, 1050, PIXMA_CAP_ADF),
   /* FIXME: the following capabilities all need updating/verifying */
   DEV ("Canon imageCLASS MF5630", "MF5630", MF5630_PID, 600, 640, 877, PIXMA_CAP_ADF),
   DEV ("Canon laserBase MF5650", "MF5650", MF5650_PID, 600, 640, 877, PIXMA_CAP_ADF),
   DEV ("Canon imageCLASS MF8170c", "MF8170c", MF8100_PID, 600, 640, 877, PIXMA_CAP_ADF),
   DEV ("Canon imageClass MF8030", "MF8030", MF8030_PID, 600, 640, 877, PIXMA_CAP_ADF),  
+  DEV ("Canon i-SENSYS MF5880dn", "MF5880", MF5880_PID, 600, 640, 877, PIXMA_CAP_ADFDUP),
+  DEV ("Canon i-SENSYS MF6680dn", "MF6680", MF6680_PID, 600, 640, 877, PIXMA_CAP_ADFDUP),
+  DEV ("Canon imageCLASS MF4570dw", "MF4570dw", MF4570_PID, 600, 640, 877, 0),
+  DEV ("Canon imageRUNNER 1133", "iR1133", IR1133_PID, 600, 637, 877, PIXMA_CAP_ADFDUP),
   DEV (NULL, NULL, 0, 0, 0, 0, 0)
 };
