@@ -2,7 +2,7 @@
 
    Copyright (C) 2007-2009 Nicolas Martin, <nicols-guest at alioth dot debian dot org>
    Copyright (C) 2006-2007 Wittawat Yamwong <wittawat@web.de>
-   Copyright (C) 2011-2012 Rolf Bensch <rolf at bensch hyphen online dot de>
+   Copyright (C) 2011-2013 Rolf Bensch <rolf at bensch hyphen online dot de>
 
    This file is part of the SANE package.
 
@@ -184,6 +184,30 @@
 #define MX710_PID 0x175D
 #define MX890_PID 0x175E
 #define E600_PID 0x175A
+#define MG4200_PID 0x1763
+
+/* 2013 new devices */
+#define MP230_PID 0x175F
+#define MG6300_PID 0x1765
+
+/* 2013 new devices (untested) */
+#define MG2200_PID 0x1760
+#define E510_PID 0x1761
+#define MG3200_PID 0x1762
+#define MG5400_PID 0x1764
+#define MX390_PID 0x1766
+#define E610_PID 0x1767
+#define MX450_PID 0x1768
+#define MX520_PID 0x1769
+#define MX720_PID 0x176a
+#define MX920_PID 0x176b
+#define MG2400_PID 0x176c
+#define MG2500_PID 0x176d
+#define MG3500_PID 0x176e
+#define MG6500_PID 0x176f
+#define MG6400_PID 0x1770
+#define MG5500_PID 0x1771
+#define MG7100_PID 0x1772
 
 
 /* Generation 4 XML messages that encapsulates the Pixma protocol messages */
@@ -739,11 +763,26 @@ send_scan_param (pixma_t * s)
                                                  * ((is_ccd_grayscale (s) || is_ccd_lineart (s)) ? 3 : s->param->channels));   /* bits per pixel */
 #endif
 
-      data[0x1f] = 0x01;
+      data[0x1f] = 0x01;        /* This one also seen at 0. Don't know yet what's used for */
       data[0x20] = 0xff;
       data[0x21] = 0x81;
       data[0x23] = 0x02;
       data[0x24] = 0x01;
+
+      switch (s->cfg->pid)
+        {
+	case MG5300_PID:
+	  /* unknown values (perhaps counter) for MG5300 series---values must be 0x30-0x39: decimal 0-9 */
+	  data[0x26] = 0x32; /* using example values from a real scan here */
+	  data[0x27] = 0x31;
+	  data[0x28] = 0x34;
+	  data[0x29] = 0x35;
+	  break;
+
+	default:
+	  break;
+	}
+
       data[0x30] = 0x01;
     }
   return pixma_exec (s, &mp->cb);
@@ -879,7 +918,7 @@ and query_status().
 static int
 handle_interrupt (pixma_t * s, int timeout)
 {
-  uint8_t buf[16];
+  uint8_t buf[64];
   int len;
 
   len = pixma_wait_interrupt (s->io, buf, sizeof (buf), timeout);
@@ -887,22 +926,47 @@ handle_interrupt (pixma_t * s, int timeout)
     return 0;
   if (len < 0)
     return len;
-  if (len != 16)
+  if (len%16)           /* len must be a multiple of 16 bytes */
     {
       PDBG (pixma_dbg
 	    (1, "WARNING:unexpected interrupt packet length %d\n", len));
       return PIXMA_EPROTO;
     }
 
-  /* More than one event can be reported at the same time. */
-  if (buf[3] & 1)
-    send_time (s);
-  if (buf[9] & 2)
-    query_status (s);
-  if (buf[0] & 2)
-    s->events = PIXMA_EV_BUTTON2 | buf[1];	/* b/w scan */
-  if (buf[0] & 1)
-    s->events = PIXMA_EV_BUTTON1 | buf[1];	/* color scan */
+  /* s->event = 0x0brroott
+   * b:  button
+   * oo: original
+   * tt: target
+   * rr: scan resolution
+   * poll event with 'scanimage -A' */
+  if (s->cfg->pid == MG6200_PID
+      || s->cfg->pid == MG6300_PID)
+  /* button no. in buf[7]
+   * size in buf[10] 01=A4; 02=Letter; 08=10x15; 09=13x18; 0b=auto
+   * format in buf[11] 01=JPEG; 02=TIFF; 03=PDF; 04=Kompakt-PDF
+   * dpi in buf[12] 01=75; 02=150; 03=300; 04=600
+   * target = format; original = size; scan-resolution = dpi */
+  {
+    if (buf[7] & 1)
+      s->events = PIXMA_EV_BUTTON1 | buf[11] | buf[10]<<8 | buf[12]<<16;    /* color scan */
+    if (buf[7] & 2)
+      s->events = PIXMA_EV_BUTTON2 | buf[11] | buf[10]<<8 | buf[12]<<16;    /* b/w scan */
+  }
+  else
+  /* button no. in buf[0]
+   * original in buf[0]
+   * target in buf[1] */
+  {
+    /* More than one event can be reported at the same time. */
+    if (buf[3] & 1)
+      send_time (s);
+    if (buf[9] & 2)
+      query_status (s);
+    if (buf[0] & 2)
+      s->events = PIXMA_EV_BUTTON2 | buf[1] | ((buf[0] & 0xf0) << 4);	/* b/w scan */
+    if (buf[0] & 1)
+      s->events = PIXMA_EV_BUTTON1 | buf[1] | ((buf[0] & 0xf0) << 4);	/* color scan */
+  }
   return 1;
 }
 
@@ -1026,7 +1090,7 @@ static unsigned
 post_process_image_data (pixma_t * s, pixma_imagebuf_t * ib)
 {
   mp150_t *mp = (mp150_t *) s->subdriver;
-  unsigned c, lines, i, line_size, n, m, cw, cx;
+  unsigned c, lines, line_size, n, m, cw, cx;
   uint8_t *sptr, *dptr, *gptr, *cptr;
 
   c = ((is_ccd_grayscale (s) || is_ccd_lineart (s)) ? 3 : s->param->channels)
@@ -1052,20 +1116,30 @@ post_process_image_data (pixma_t * s, pixma_imagebuf_t * ib)
 	           lines, 2 * mp->color_shift + mp->stripe_shift));*/
   if (lines > 2 * mp->color_shift + mp->stripe_shift)
     {
+      unsigned i;
+
       lines -= 2 * mp->color_shift + mp->stripe_shift;
       for (i = 0; i < lines; i++, sptr += line_size)
         {
           /* Color plane and stripes shift needed by e.g. CCD */
           /*PDBG (pixma_dbg (4, "*post_process_image_data***** Processing with c=%u, n=%u, m=%u, w=%i, line_size=%u ***** \n",
 	        c, n, m, s->param->wx, line_size));*/
-          if (c >= 3)
+          if (s->cfg->pid != MG5300_PID && s->cfg->pid != MG6300_PID && c >= 3)
             dptr = shift_colors (dptr, sptr, 
                                  s->param->wx, s->param->xdpi, s->cfg->pid, c,
                                  mp->shift, mp->stripe_shift);
                        
           /* special image format for *most* devices at high dpi. 
-           * MP220 and MX360 are exceptions */
-          if (s->cfg->pid != MP220_PID && s->cfg->pid != MX360_PID && n > 0)
+           * MP220, MX360, MX370, MX890, MG5300 are exceptions */
+          if (n > 0
+              && s->cfg->pid != MP220_PID
+              && s->cfg->pid != MX360_PID
+              && s->cfg->pid != MX370_PID
+              && s->cfg->pid != MX890_PID
+              && s->cfg->pid != MG3100_PID
+              && s->cfg->pid != MG2100_PID
+              && s->cfg->pid != MG5300_PID
+              && s->cfg->pid != MG6300_PID)
               reorder_pixels (mp->linebuf, sptr, c, n, m, s->param->wx, line_size);
           
           /* Crop line to selected borders */
@@ -1155,7 +1229,6 @@ static int
 mp150_check_param (pixma_t * s, pixma_scan_param_t * sp)
 {
   mp150_t *mp = (mp150_t *) s->subdriver;
-  unsigned w_max;
 
   /* PDBG (pixma_dbg (4, "*mp150_check_param***** Initially: channels=%u, depth=%u, x=%u, y=%u, w=%u, h=%u, xs=%u, wx=%u *****\n",
                    sp->channels, sp->depth, sp->x, sp->y, sp->w, sp->h, sp->xs, sp->wx)); */
@@ -1183,6 +1256,8 @@ mp150_check_param (pixma_t * s, pixma_scan_param_t * sp)
   /* for software lineart w must be a multiple of 8 */
   if (sp->software_lineart == 1 && sp->w % 8)
     {
+      unsigned w_max;
+
       sp->w += 8 - (sp->w % 8);
 
       /* do not exceed the scanner capability */
@@ -1216,10 +1291,12 @@ mp150_check_param (pixma_t * s, pixma_scan_param_t * sp)
         s->cfg->pid == MX340_PID ||
         s->cfg->pid == MX350_PID ||
         s->cfg->pid == MX360_PID ||
+        s->cfg->pid == MX410_PID ||
+        s->cfg->pid == MX420_PID ||
         s->cfg->pid == MX7600_PID )
        &&
         sp->source == PIXMA_SOURCE_FLATBED)
-    sp->h = MIN (sp->h, 877 * sp->ydpi / 75);
+    sp->h = MIN (sp->h, 877 * sp->xdpi / 75);
     
   if (sp->source == PIXMA_SOURCE_TPU)
     {
@@ -1265,7 +1342,7 @@ mp150_check_param (pixma_t * s, pixma_scan_param_t * sp)
 static int
 mp150_scan (pixma_t * s)
 {
-  int error = 0, tmo, i;
+  int error = 0, tmo;
   mp150_t *mp = (mp150_t *) s->subdriver;
 
   if (mp->state != state_idle)
@@ -1361,8 +1438,12 @@ mp150_scan (pixma_t * s)
       if ((error >= 0) && (mp->generation >= 3) && has_ccd_sensor (s))
         error = init_ccd_lamp_3 (s);
       if ((error >= 0) && !is_scanning_from_tpu (s))
-        for (i = (mp->generation >= 3) ? 3 : 1 ; i > 0 && error >= 0; i--)
-          error = send_gamma_table (s);
+        {
+          int i;
+
+          for (i = (mp->generation >= 3) ? 3 : 1 ; i > 0 && error >= 0; i--)
+            error = send_gamma_table (s);
+        }
       else if (error >= 0)  /* in TPU mode, for gen 1, 2, and 3 */
         error = send_set_tpu_info (s);
     }
@@ -1540,14 +1621,15 @@ static const pixma_scan_ops_t pixma_mp150_ops = {
   mp150_get_status
 };
 
-#define DEVICE(name, model, pid, dpi, ext_min_dpi, ext_max_dpi, w, h, cap) { \
+#define DEVICE(name, model, pid, dpi, adftpu_min_dpi, adftpu_max_dpi, w, h, cap) { \
         name,              /* name */               \
         model,             /* model */              \
         CANON_VID, pid,    /* vid pid */            \
         0,                 /* iface */              \
         &pixma_mp150_ops,  /* ops */                \
         dpi, 2*(dpi),      /* xdpi, ydpi */         \
-        ext_min_dpi, ext_max_dpi,   /* ext_min_dpi, ext_max_dpi */ \
+        adftpu_min_dpi, adftpu_max_dpi,         /* adftpu_min_dpi, adftpu_max_dpi */ \
+        0, 0,              /* tpuir_min_dpi & tpuir_max_dpi not used in this subdriver */  \
         w, h,              /* width, height */      \
         PIXMA_CAP_EASY_RGB|                         \
         PIXMA_CAP_GRAY|    /* CIS with native grayscale and CCD with software grayscale */ \
@@ -1632,7 +1714,7 @@ const pixma_config_t pixma_mp150_devices[] = {
   DEVICE ("Canon PIXMA MP495",  "MP495",  MP495_PID,  1200, 0, 0, 638, 877, PIXMA_CAP_CIS),
   DEVICE ("Canon PIXMA MG5100", "MG5100", MG5100_PID, 1200, 0, 0, 638, 877, PIXMA_CAP_CIS),
   DEVICE ("Canon PIXMA MG5200", "MG5200", MG5200_PID, 2400, 0, 0, 638, 877, PIXMA_CAP_CIS),
-  DEVICE ("Canon PIXMA MG6100", "MG6200", MG6100_PID, 2400, 0, 0, 638, 877, PIXMA_CAP_CIS),
+  DEVICE ("Canon PIXMA MG6100", "MG6100", MG6100_PID, 2400, 0, 0, 638, 877, PIXMA_CAP_CIS),
 
   /* Latest devices (2011) Generation 4 CIS/CCD */
   DEVICE ("Canon PIXMA MG2100", "MG2100", MG2100_PID, 1200, 0, 0, 638, 877, PIXMA_CAP_CIS),
@@ -1641,15 +1723,37 @@ const pixma_config_t pixma_mp150_devices[] = {
   DEVICE ("Canon PIXMA MG5300", "MG5300", MG5300_PID, 2400, 0, 0, 638, 877, PIXMA_CAP_CIS),
   DEVICE ("Canon PIXMA MG6200", "MG6200", MG6200_PID, 4800, 0, 0, 638, 877, PIXMA_CAP_CIS),
   DEVICE ("Canon PIXMA MP493",  "MP493",  MP493_PID,  1200, 0, 0, 638, 877, PIXMA_CAP_CIS),
-  DEVICE ("Canon PIXMA E500",   "E500",   E500_PID,   1200, 0, 0, 638, 877, PIXMA_CAP_CIS | PIXMA_CAP_EXPERIMENT),
+  DEVICE ("Canon PIXMA E500",   "E500",   E500_PID,   1200, 0, 0, 638, 877, PIXMA_CAP_CIS),
 
   /* Latest devices (2012) Generation 4 CIS */
-  DEVICE ("Canon PIXMA MX370 Series", "MX370", MX370_PID, 1200, 0, 0, 638, 877, PIXMA_CAP_CIS | PIXMA_CAP_ADF | PIXMA_CAP_EXPERIMENT),
-  DEVICE ("Canon PIXMA MX430 Series", "MX430", MX430_PID, 1200, 0, 0, 638, 877, PIXMA_CAP_CIS | PIXMA_CAP_ADF | PIXMA_CAP_EXPERIMENT),
-  DEVICE ("Canon PIXMA MX510 Series", "MX510", MX510_PID, 1200, 0, 0, 638, 877, PIXMA_CAP_CIS | PIXMA_CAP_ADF | PIXMA_CAP_EXPERIMENT),
-  DEVICE ("Canon PIXMA MX710 Series", "MX710", MX710_PID, 2400, 0, 0, 638, 877, PIXMA_CAP_CIS | PIXMA_CAP_ADFDUP | PIXMA_CAP_EXPERIMENT),
-  DEVICE ("Canon PIXMA MX890 Series", "MX890", MX890_PID, 2400, 0, 0, 638, 877, PIXMA_CAP_CIS | PIXMA_CAP_ADFDUP | PIXMA_CAP_EXPERIMENT),
-  DEVICE ("Canon PIXMA E600 Series",  "E600",  E600_PID,  1200, 0, 0, 638, 877, PIXMA_CAP_CIS | PIXMA_CAP_ADF | PIXMA_CAP_EXPERIMENT),
+  DEVICE ("Canon PIXMA MX370 Series", "MX370", MX370_PID, 1200, 0, 0, 638, 877, PIXMA_CAP_CIS | PIXMA_CAP_ADF),
+  DEVICE ("Canon PIXMA MX430 Series", "MX430", MX430_PID, 1200, 0, 0, 638, 877, PIXMA_CAP_CIS | PIXMA_CAP_ADF),
+  DEVICE ("Canon PIXMA MX510 Series", "MX510", MX510_PID, 1200, 0, 0, 638, 877, PIXMA_CAP_CIS | PIXMA_CAP_ADF),
+  DEVICE ("Canon PIXMA MX710 Series", "MX710", MX710_PID, 2400, 0, 0, 638, 877, PIXMA_CAP_CIS | PIXMA_CAP_ADFDUP),
+  DEVICE ("Canon PIXMA MX890 Series", "MX890", MX890_PID, 2400, 0, 0, 638, 877, PIXMA_CAP_CIS | PIXMA_CAP_ADFDUP),
+  DEVICE ("Canon PIXMA E600 Series",  "E600",  E600_PID,  1200, 0, 0, 638, 877, PIXMA_CAP_CIS | PIXMA_CAP_ADF),
+  DEVICE ("Canon PIXMA MG4200", "MG4200", MG4200_PID, 1200, 0, 0, 638, 877, PIXMA_CAP_CIS),
+
+  /* Latest devices (2013) Generation 4 CIS */
+  DEVICE ("Canon PIXMA E510",  "E510",  E510_PID,  1200, 0, 0, 638, 877, PIXMA_CAP_CIS),
+  DEVICE ("Canon PIXMA E610",  "E610",  E610_PID,  1200, 0, 0, 638, 877, PIXMA_CAP_CIS | PIXMA_CAP_ADF),
+  DEVICE ("Canon PIXMA MP230", "MP230", MP230_PID, 600, 0, 0, 638, 877, PIXMA_CAP_CIS),
+  DEVICE ("Canon PIXMA MG2200 Series", "MG2200", MG2200_PID, 1200, 0, 0, 638, 877, PIXMA_CAP_CIS),
+  DEVICE ("Canon PIXMA MG3200 Series", "MG3200", MG3200_PID, 1200, 0, 0, 638, 877, PIXMA_CAP_CIS),
+  DEVICE ("Canon PIXMA MG5400 Series", "MG5400", MG5400_PID, 2400, 0, 0, 638, 877, PIXMA_CAP_CIS),
+  DEVICE ("Canon PIXMA MG6300 Series", "MG6300", MG6300_PID, 2400, 0, 0, 638, 877, PIXMA_CAP_CIS),
+  DEVICE ("Canon PIXMA MX390 Series", "MX390", MX390_PID, 1200, 0, 0, 638, 877, PIXMA_CAP_CIS | PIXMA_CAP_ADF),
+  DEVICE ("Canon PIXMA MX450 Series", "MX450", MX450_PID, 1200, 0, 0, 638, 877, PIXMA_CAP_CIS | PIXMA_CAP_ADF),
+  DEVICE ("Canon PIXMA MX520 Series", "MX520", MX520_PID, 1200, 0, 0, 638, 877, PIXMA_CAP_CIS | PIXMA_CAP_ADF),
+  DEVICE ("Canon PIXMA MX720 Series", "MX720", MX720_PID, 2400, 0, 0, 638, 877, PIXMA_CAP_CIS | PIXMA_CAP_ADF),
+  DEVICE ("Canon PIXMA MX920 Series", "MX920", MX920_PID, 2400, 0, 600, 638, 877, PIXMA_CAP_CIS | PIXMA_CAP_ADFDUP),
+  DEVICE ("Canon PIXMA MG2400 Series", "MG2400", MG2400_PID, 600, 0, 0, 638, 877, PIXMA_CAP_CIS),
+  DEVICE ("Canon PIXMA MG2500 Series", "MG2500", MG2500_PID, 600, 0, 0, 638, 877, PIXMA_CAP_CIS),
+  DEVICE ("Canon PIXMA MG3500 Series", "MG3500", MG3500_PID, 1200, 0, 0, 638, 877, PIXMA_CAP_CIS),
+  DEVICE ("Canon PIXMA MG5500 Series", "MG5500", MG5500_PID, 1200, 0, 0, 638, 877, PIXMA_CAP_CIS),
+  DEVICE ("Canon PIXMA MG6400 Series", "MG6400", MG6400_PID, 1200, 0, 0, 638, 877, PIXMA_CAP_CIS),
+  DEVICE ("Canon PIXMA MG6500 Series", "MG6500", MG6500_PID, 2400, 0, 0, 638, 877, PIXMA_CAP_CIS),
+  DEVICE ("Canon PIXMA MG7100 Series", "MG7100", MG7100_PID, 2400, 0, 0, 638, 877, PIXMA_CAP_CIS),
 
   END_OF_DEVICE_LIST
 };
