@@ -70,7 +70,7 @@
 # include "lalloca.h"		/* MUST come first for AIX! */
 #endif
 
-#include "sane/config.h"
+#include "../include/sane/config.h"
 #include "lalloca.h"
 
 #include <assert.h>
@@ -88,19 +88,19 @@
 #include <sys/wait.h>
 #include <unistd.h>
 
-#include "sane/sane.h"
-#include "sane/sanei.h"
-#include "sane/saneopts.h"
+#include "../include/sane/sane.h"
+#include "../include/sane/sanei.h"
+#include "../include/sane/saneopts.h"
 
 
 #define BACKEND_NAME qcam
-#include "sane/sanei_backend.h"
+#include "../include/sane/sanei_backend.h"
 
 #ifndef PATH_MAX
 # define PATH_MAX	1024
 #endif
 
-#include "sane/sanei_config.h"
+#include "../include/sane/sanei_config.h"
 #define QCAM_CONFIG_FILE "qcam.conf"
 
 #include "qcam.h"
@@ -193,23 +193,24 @@ static const SANE_Range odd_bw_x_range = { 1, 335, 2 };
 static const SANE_Range bw_y_range = { 0, 241, 1 };
 static const SANE_Range odd_bw_y_range = { 1, 242, 1 };
 
-#if defined(__linux__) || defined (HAVE_SYS_HW_H)
+#if defined(HAVE_SYS_IO_H) || defined(HAVE_ASM_IO_H) || defined (HAVE_SYS_HW_H)
 
 #ifdef HAVE_SYS_IO_H
-# include <sys/io.h>		/* GNU libc based Linux */
+# include <sys/io.h>		/* GNU libc based OS */
 #elif HAVE_ASM_IO_H
 # include <asm/io.h>		/* older Linux */
 #elif HAVE_SYS_HW_H
 # include <sys/hw.h>		/* OS/2 */
 #endif
 
+#endif /* <sys/io.h> || <asm/io.h> || <sys/hw.h> */
+
 #define read_lpdata(d)		inb ((d)->port)
 #define read_lpstatus(d)	inb ((d)->port + 1)
 #define read_lpcontrol(d)	inb ((d)->port + 2)
 #define write_lpdata(d,v)	outb ((v), (d)->port)
 #define write_lpcontrol(d,v)	outb ((v), (d)->port + 2)
 
-#endif /* __linux__ */
 
 static SANE_Status
 enable_ports (QC_Device * q)
@@ -1254,8 +1255,7 @@ init_options (QC_Scanner * s)
   for (i = 0; i < NUM_OPTIONS; ++i)
     {
       s->opt[i].size = sizeof (SANE_Word);
-      s->opt[i].cap = (SANE_CAP_SOFT_SELECT | SANE_CAP_SOFT_DETECT
-		       | SANE_CAP_ALWAYS_SETTABLE);
+      s->opt[i].cap = (SANE_CAP_SOFT_SELECT | SANE_CAP_SOFT_DETECT);
     }
 
   s->opt[OPT_NUM_OPTS].title = SANE_TITLE_NUM_OPTIONS;
@@ -1463,7 +1463,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
   DBG (1, "sane_init: enter\n");
 
   if (version_code)
-    *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, 0);
+    *version_code = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, 0);
 
   fp = sanei_config_open (QCAM_CONFIG_FILE);
   if (!fp)
