@@ -1,7 +1,7 @@
 #ifndef SP15C_H
 #define SP15C_H
 
-static const char RCSid_h[] = "$Header: /cvsroot/sane/sane-backends/backend/sp15c.h,v 1.7 2005-09-19 19:57:48 fzago-guest Exp $";
+static const char RCSid_h[] = "$Header$";
 /* sane - Scanner Access Now Easy.
 
    This file is part of the SANE package.
@@ -48,7 +48,10 @@ static const char RCSid_h[] = "$Header: /cvsroot/sane/sane-backends/backend/sp15
 
 /* ------------------------------------------------------------------------- */
 /*
- * $Log: sp15c.h,v $
+ * $Log$
+ * Revision 1.8  2008/05/15 12:50:24  ellert-guest
+ * Fix for bug #306751: sanei-thread with pthreads on 64 bit
+ *
  * Revision 1.7  2005-09-19 19:57:48  fzago-guest
  * Replaced __unused__ with __sane_unused__ to avoid a namespace conflict.
  *
@@ -147,7 +150,7 @@ struct sp15c
     int scanning;		/* "in progress" flag */
     int autofeeder;		/* detected */
     int use_adf;		/* requested */
-    int reader_pid;		/* child is running */
+    SANE_Pid reader_pid;	/* child is running */
     int prescan;		/* ??? */
 
 /***** terms for "set window" command *****/
@@ -189,7 +192,6 @@ struct sp15c
 
 /* ------------------------------------------------------------------------- */
 
-#define MM_PER_INCH	25.4
 #define length_quant SANE_UNFIX(SANE_FIX(MM_PER_INCH / 1200.0))
 #define mmToIlu(mm) ((mm) / length_quant)
 #define iluToMm(ilu) ((ilu) * length_quant)
