@@ -1,5 +1,5 @@
 /* sane - Scanner Access Now Easy.
-   Copyright (C) 2001 Stphane Voltz <svoltz@wanadoo.fr>
+   Copyright (C) 2001-2012 Stphane Voltz <stef.dev@free.fr>
    This file is part of the SANE package.
 
    This program is free software; you can redistribute it and/or
@@ -174,6 +174,25 @@ typedef struct Umax_PP_Device
 Umax_PP_Device;
 
 
+/**
+ * enumeration of configuration options
+ */
+enum Umax_PP_Configure_Option
+{
+  CFG_BUFFER = 0,
+  CFG_RED_GAIN,
+  CFG_GREEN_GAIN,
+  CFG_BLUE_GAIN,
+  CFG_RED_OFFSET,
+  CFG_GREEN_OFFSET,
+  CFG_BLUE_OFFSET,
+  CFG_VENDOR,
+  CFG_NAME,
+  CFG_MODEL,
+  CFG_ASTRA,
+  NUM_CFG_OPTIONS
+};
+
 #if (!defined __GNUC__ || __GNUC__ < 2 || \
      __GNUC_MINOR__ < (defined __cplusplus ? 6 : 4))
 
@@ -182,7 +201,7 @@ Umax_PP_Device;
 #endif
 
 #define DEBUG()		DBG(4, "%s(v%d.%d.%d-%s): line %d: debug exception\n", \
-			  __PRETTY_FUNCTION__, V_MAJOR, V_MINOR,	\
+			  __PRETTY_FUNCTION__, SANE_CURRENT_MAJOR, V_MINOR,	\
 			  UMAX_PP_BUILD, UMAX_PP_STATE, __LINE__)
 
 #endif /* umax_pp_h */
