@@ -1,5 +1,5 @@
 /* sane - Scanner Access Now Easy.
-   Copyright (C) 2001 Stphane Voltz <svoltz@wanadoo.fr>
+   Copyright (C) 2001-2012 Stphane Voltz <stef.dev@free.fr>
    This file is part of the SANE package.
 
    This program is free software; you can redistribute it and/or
@@ -40,6 +40,10 @@
 
    This file implements a SANE backend for Umax PP flatbed scanners.  */
 
+#define DEBUG_DECLARE_ONLY
+#undef BACKEND_NAME
+#define BACKEND_NAME umax_pp
+
 #include "../include/sane/config.h"
 #include <stdlib.h>
 #include <string.h>
@@ -78,14 +82,14 @@ lock_parport (void)
   if ((fd > 0) && (!locked))
     {
       if (ioctl (sanei_umax_pp_getparport (), PPCLAIM))
-	{
-	  return (UMAX1220P_BUSY);
-	}
+        {
+          return UMAX1220P_BUSY;
+        }
 #ifdef PPGETMODE
       if (ioctl (fd, PPGETMODE, &exmode))
-	exmode = IEEE1284_MODE_COMPAT;
+        exmode = IEEE1284_MODE_COMPAT;
       if (ioctl (fd, PPGETFLAGS, &exflags))
-	exflags = 0;
+        exflags = 0;
 #endif
       mode = IEEE1284_MODE_EPP;
       ioctl (fd, PPNEGOT, &mode);
@@ -95,7 +99,7 @@ lock_parport (void)
 #else
   locked = 1;
 #endif
-  return (UMAX1220P_OK);
+  return UMAX1220P_OK;
 }
 
 
@@ -121,7 +125,7 @@ unlock_parport (void)
 #endif
   DBG (3, "unlock_parport\n");
   locked = 0;
-  return (UMAX1220P_OK);
+  return UMAX1220P_OK;
 }
 
 
@@ -143,7 +147,7 @@ sanei_umax_pp_model (int port, int *model)
   DBG (3, "sanei_umax_pp_model\n");
   sanei_umax_pp_setport (port);
   if (lock_parport () == UMAX1220P_BUSY)
-    return (UMAX1220P_BUSY);
+    return UMAX1220P_BUSY;
 
   /* init transport layer */
   /* 0: failed
@@ -153,45 +157,45 @@ sanei_umax_pp_model (int port, int *model)
    */
   do
     {
-      rc = sanei_umax_pp_InitTransport (recover);
+      rc = sanei_umax_pp_initTransport (recover);
     }
   while (rc == 2);
 
   if (rc == 3)
     {
       unlock_parport ();
-      return (UMAX1220P_BUSY);
+      return UMAX1220P_BUSY;
     }
   if (rc != 1)
     {
-      DBG (0, "sanei_umax_pp_InitTransport() failed (%s:%d)\n", __FILE__,
-	   __LINE__);
+      DBG (0, "sanei_umax_pp_initTransport() failed (%s:%d)\n", __FILE__,
+           __LINE__);
       unlock_parport ();
-      return (UMAX1220P_TRANSPORT_FAILED);
+      return UMAX1220P_TRANSPORT_FAILED;
     }
 
   /* check model only, and if only none given in conf file */
   if (!sanei_umax_pp_getastra ())
     {
-      rc = sanei_umax_pp_CheckModel ();
+      rc = sanei_umax_pp_checkModel ();
     }
   else
     {
       rc = sanei_umax_pp_getastra ();
     }
-  sanei_umax_pp_EndSession ();
+  sanei_umax_pp_endSession ();
   unlock_parport ();
-  if (rc < 610)
+  if (rc < 600)
     {
       DBG (0, "sanei_umax_pp_CheckModel() failed (%s:%d)\n", __FILE__,
-	   __LINE__);
-      return (UMAX1220P_PROBE_FAILED);
+           __LINE__);
+      return UMAX1220P_PROBE_FAILED;
     }
   *model = rc;
 
 
   /* OK */
-  return (UMAX1220P_OK);
+  return UMAX1220P_OK;
 }
 
 int
@@ -210,37 +214,37 @@ sanei_umax_pp_attach (int port, char *name)
     }
 
   sanei_umax_pp_setport (port);
-  if (sanei_umax_pp_InitPort (port, name) != 1)
-    return (UMAX1220P_PROBE_FAILED);
+  if (sanei_umax_pp_initPort (port, name) != 1)
+    return UMAX1220P_PROBE_FAILED;
 
   /* init port locks the port, so we flag that */
   locked = 1;
 
-  if (sanei_umax_pp_ProbeScanner (recover) != 1)
+  if (sanei_umax_pp_probeScanner (recover) != 1)
     {
       if (recover)
-	{
-	  sanei_umax_pp_InitTransport (recover);
-	  sanei_umax_pp_EndSession ();
-	  if (sanei_umax_pp_ProbeScanner (recover) != 1)
-	    {
-	      DBG (0, "Recover failed ....\n");
-	      unlock_parport ();
-	      return (UMAX1220P_PROBE_FAILED);
-	    }
-	}
+        {
+          sanei_umax_pp_initTransport (recover);
+          sanei_umax_pp_endSession ();
+          if (sanei_umax_pp_probeScanner (recover) != 1)
+            {
+              DBG (0, "Recover failed ....\n");
+              unlock_parport ();
+              return UMAX1220P_PROBE_FAILED;
+            }
+        }
       else
-	{
-	  unlock_parport ();
-	  return (UMAX1220P_PROBE_FAILED);
-	}
+        {
+          unlock_parport ();
+          return UMAX1220P_PROBE_FAILED;
+        }
     }
-  sanei_umax_pp_EndSession ();
+  sanei_umax_pp_endSession ();
   unlock_parport ();
 
 
   /* OK */
-  return (UMAX1220P_OK);
+  return UMAX1220P_OK;
 }
 
 
@@ -257,7 +261,7 @@ sanei_umax_pp_open (int port, char *name)
     sanei_umax_pp_setport (port);
 
   if (lock_parport () == UMAX1220P_BUSY)
-    return (UMAX1220P_BUSY);
+    return UMAX1220P_BUSY;
 
   /* init transport layer */
   /* 0: failed
@@ -267,37 +271,37 @@ sanei_umax_pp_open (int port, char *name)
    */
   do
     {
-      rc = sanei_umax_pp_InitTransport (recover);
+      rc = sanei_umax_pp_initTransport (recover);
     }
   while (rc == 2);
 
   if (rc == 3)
     {
       unlock_parport ();
-      return (UMAX1220P_BUSY);
+      return UMAX1220P_BUSY;
     }
 
   if (rc != 1)
     {
 
-      DBG (0, "sanei_umax_pp_InitTransport() failed (%s:%d)\n", __FILE__,
-	   __LINE__);
+      DBG (0, "sanei_umax_pp_initTransport() failed (%s:%d)\n", __FILE__,
+           __LINE__);
       unlock_parport ();
-      return (UMAX1220P_TRANSPORT_FAILED);
+      return UMAX1220P_TRANSPORT_FAILED;
     }
   /* init scanner */
-  if (sanei_umax_pp_InitScanner (recover) == 0)
+  if (sanei_umax_pp_initScanner (recover) == 0)
     {
-      DBG (0, "sanei_umax_pp_InitScanner() failed (%s:%d)\n", __FILE__,
-	   __LINE__);
-      sanei_umax_pp_EndSession ();
+      DBG (0, "sanei_umax_pp_initScanner() failed (%s:%d)\n", __FILE__,
+           __LINE__);
+      sanei_umax_pp_endSession ();
       unlock_parport ();
-      return (UMAX1220P_SCANNER_FAILED);
+      return UMAX1220P_SCANNER_FAILED;
     }
 
   /* OK */
   unlock_parport ();
-  return (UMAX1220P_OK);
+  return UMAX1220P_OK;
 }
 
 
@@ -306,40 +310,39 @@ sanei_umax_pp_cancel (void)
 {
   DBG (3, "sanei_umax_pp_cancel\n");
   if (lock_parport () == UMAX1220P_BUSY)
-    return (UMAX1220P_BUSY);
+    return UMAX1220P_BUSY;
 
-  /* maybe bus reset here if exists */
-  sanei_umax_pp_CmdSync (0xC2);
-  sanei_umax_pp_CmdSync (0x00);
-  sanei_umax_pp_CmdSync (0x00);
-  if (sanei_umax_pp_Park () == 0)
+  /* maybe EPAT reset here if exists */
+  sanei_umax_pp_cmdSync (0xC2);
+  sanei_umax_pp_cmdSync (0x00);
+  sanei_umax_pp_cmdSync (0x00);
+  if (sanei_umax_pp_park () == 0)
     {
-      DBG (0, "Park failed !!! (%s:%d)\n", __FILE__, __LINE__);
+      DBG (0, "sanei_umax_pp_park failed !!! (%s:%d)\n", __FILE__, __LINE__);
       unlock_parport ();
-      return (UMAX1220P_PARK_FAILED);
+      return UMAX1220P_PARK_FAILED;
     }
-  /* EndSession() cancels any pending command  */
+  /* endSession() cancels any pending command  */
   /* such as parking ...., so we only return   */
-  sanei_umax_pp_ReleaseScanner ();
   unlock_parport ();
-  return (UMAX1220P_OK);
+  return UMAX1220P_OK;
 }
 
 
 
 int
 sanei_umax_pp_start (int x, int y, int width, int height, int dpi, int color,
-		     int autoset,
-		     int brightness, int contrast, int *rbpp, int *rtw,
-		     int *rth)
+                     int autoset,
+                     int gain, int offset, int *rbpp, int *rtw,
+                     int *rth)
 {
   int col = BW_MODE;
 
   DBG (3, "sanei_umax_pp_start\n");
   if (lock_parport () == UMAX1220P_BUSY)
-    return (UMAX1220P_BUSY);
+    return UMAX1220P_BUSY;
   /* end session isn't done by cancel any more */
-  sanei_umax_pp_EndSession ();
+  sanei_umax_pp_endSession ();
 
   if (autoset)
     sanei_umax_pp_setauto (1);
@@ -359,32 +362,46 @@ sanei_umax_pp_start (int x, int y, int width, int height, int dpi, int color,
       break;
     }
 
-  if (sanei_umax_pp_StartScan
-      (x + 144, y, width, height, dpi, col, brightness, contrast, rbpp, rtw,
-       rth) != 1)
+  if (sanei_umax_pp_startScan
+      (x + sanei_umax_pp_getLeft (), y, width, height, dpi, col, gain,
+       offset, rbpp, rtw, rth) != 1)
     {
-      sanei_umax_pp_EndSession ();
+      sanei_umax_pp_endSession ();
       unlock_parport ();
-      return (UMAX1220P_START_FAILED);
+      return UMAX1220P_START_FAILED;
     }
   unlock_parport ();
-  return (UMAX1220P_OK);
+  return UMAX1220P_OK;
 }
 
 int
 sanei_umax_pp_read (long len, int window, int dpi, int last,
-		    unsigned char *buffer)
+                    unsigned char *buffer)
 {
+  int read = 0;
+  int bytes;
+
   DBG (3, "sanei_umax_pp_read\n");
   if (lock_parport () == UMAX1220P_BUSY)
-    return (UMAX1220P_BUSY);
-  if (sanei_umax_pp_ReadBlock (len, window, dpi, last, buffer) == 0)
+    return UMAX1220P_BUSY;
+
+  /* since 610P may override len and last to meet its */
+  /* hardware requirements, we have to loop until we  */
+  /* have all the data                                */
+  while (read < len)
     {
-      sanei_umax_pp_EndSession ();
-      return (UMAX1220P_READ_FAILED);
+      bytes =
+        sanei_umax_pp_readBlock (len - read, window, dpi, last,
+                                 buffer + read);
+      if (bytes == 0)
+        {
+          sanei_umax_pp_endSession ();
+          return UMAX1220P_READ_FAILED;
+        }
+      read += bytes;
     }
   unlock_parport ();
-  return (UMAX1220P_OK);
+  return UMAX1220P_OK;
 }
 
 
@@ -394,16 +411,21 @@ sanei_umax_pp_lamp (int on)
 {
   /* init transport layer */
   DBG (3, "sanei_umax_pp_lamp\n");
-  if (lock_parport () == UMAX1220P_BUSY)
-    return (UMAX1220P_BUSY);
 
-  if (sanei_umax_pp_SetLamp (on) == 0)
+  /* no lamp support for 610P ... */
+  if (sanei_umax_pp_getastra () < 1210)
+    return UMAX1220P_OK;
+
+  if (lock_parport () == UMAX1220P_BUSY)
+    return UMAX1220P_BUSY;
+
+  if (sanei_umax_pp_setLamp (on) == 0)
     {
       DBG (0, "Setting lamp state failed!\n");
     }
 
   unlock_parport ();
-  return (UMAX1220P_OK);
+  return UMAX1220P_OK;
 }
 
 
@@ -416,18 +438,16 @@ sanei_umax_pp_status (void)
 
   DBG (3, "sanei_umax_pp_status\n");
   if (lock_parport () == UMAX1220P_BUSY)
-    return (UMAX1220P_BUSY);
+    return UMAX1220P_BUSY;
   /* check if head is at home */
-  sanei_umax_pp_CmdSync (0x40);
-  status = sanei_umax_pp_ScannerStatus ();
-  if ((status & MOTOR_BIT) == 0x00)
-    {
-      unlock_parport ();
-      return (UMAX1220P_BUSY);
-    }
+  sanei_umax_pp_cmdSync (0x40);
+  status = sanei_umax_pp_scannerStatus ();
   unlock_parport ();
+  DBG (8, "sanei_umax_pp_status=0x%02X\n", status);
+  if (((status & ASIC_BIT) != 0x00)||((status & MOTOR_BIT) == 0x00))
+    return UMAX1220P_BUSY;
 
-  return (UMAX1220P_OK);
+  return UMAX1220P_OK;
 }
 
 int
@@ -440,7 +460,7 @@ sanei_umax_pp_close ()
   DBG (3, "sanei_umax_pp_close\n");
 
   lock_parport ();
-  sanei_umax_pp_EndSession ();
+  sanei_umax_pp_endSession ();
   unlock_parport ();
 
 #ifdef HAVE_LINUX_PPDEV_H
@@ -451,5 +471,5 @@ sanei_umax_pp_close ()
       sanei_umax_pp_setparport (0);
     }
 #endif
-  return (UMAX1220P_OK);
+  return UMAX1220P_OK;
 }
