@@ -1,6 +1,6 @@
 /* sane-find-scanner.c
 
-   Copyright (C) 1997-2005 Oliver Rauch, Henning Meier-Geinitz, and others.
+   Copyright (C) 1997-2013 Oliver Rauch, Henning Meier-Geinitz, and others.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -57,6 +57,10 @@ extern char * check_usb_chip (struct usb_device *dev, int verbosity, SANE_Bool f
 
 #ifdef HAVE_LIBUSB_1_0
 #include <libusb.h>
+extern char * check_usb_chip (int verbosity,
+			      struct libusb_device_descriptor desc,
+			      libusb_device_handle *hdl,
+			      struct libusb_config_descriptor *config0);
 #endif
 
 #include "../include/sane/sanei_usb.h"
@@ -1020,7 +1024,6 @@ check_libusb_device (libusb_device *dev, SANE_Bool from_file)
 	}
     }
 
-  libusb_close (hdl);
 
   /* Some heuristics, which device may be a scanner */
   if (desc.idVendor == 0)	/* hub */
@@ -1046,7 +1049,7 @@ check_libusb_device (libusb_device *dev, SANE_Bool from_file)
 	    break;
 	  case USB_CLASS_PER_INTERFACE:
 	    if ((config0->interface[intf].num_altsetting == 0)
-		|| config0->interface[intf].altsetting)
+		|| !config0->interface[intf].altsetting)
 	      break;
 	    switch (config0->interface[intf].altsetting[0].bInterfaceClass)
 	      {
@@ -1060,13 +1063,12 @@ check_libusb_device (libusb_device *dev, SANE_Bool from_file)
 	}
     }
 
-  libusb_free_config_descriptor (config0);
-
   if (is_scanner > 0)
     {
-#if 0
-      char *chipset = check_usb_chip (dev, verbose, from_file);
-#endif
+      char *chipset = NULL;
+      
+      if(!from_file)
+        chipset = check_usb_chip (verbose, desc, hdl, config0);
 
       printf ("found USB scanner (vendor=0x%04x", vid);
       if (vendor)
@@ -1074,10 +1076,8 @@ check_libusb_device (libusb_device *dev, SANE_Bool from_file)
       printf (", product=0x%04x", pid);
       if (product)
 	printf (" [%s]", product);
-#if 0
       if (chipset)
 	printf (", chip=%s", chipset);
-#endif
       if (from_file)
 	printf (")\n");
       else
@@ -1086,8 +1086,11 @@ check_libusb_device (libusb_device *dev, SANE_Bool from_file)
       libusb_device_found = SANE_TRUE;
       device_found = SANE_TRUE;
     }
+  
+  libusb_free_config_descriptor (config0);
 
  out_free:
+  libusb_close (hdl);
   if (vendor)
     free (vendor);
 
@@ -1991,9 +1994,6 @@ main (int argc, char **argv)
 	if (verbose > 2)
 	  printf ("trying libusb:\n");
 
-	if (verbose > 3)
-	  libusb_set_debug (sfs_usb_ctx, 3);
-
 	ret = libusb_init (&sfs_usb_ctx);
 	if (ret < 0)
 	  {
@@ -2003,6 +2003,9 @@ main (int argc, char **argv)
 	    goto failed_libusb_1_0;
 	  }
 
+	if (verbose > 3)
+	  libusb_set_debug (sfs_usb_ctx, 3);
+
 	devcnt = libusb_get_device_list (sfs_usb_ctx, &devlist);
 	if (devcnt < 0)
 	  {
